@@ -103,6 +103,9 @@ is.col_spec <- function(x) inherits(x, "col_spec")
 as.col_spec <- function(x) UseMethod("as.col_spec")
 #' @export
 as.col_spec.character <- function(x) {
+  if (is_named(x)) {
+    return(as.col_spec(as.list(x)))
+  }
   letters <- strsplit(x, "")[[1]]
   col_spec(lapply(letters, col_concise), col_guess())
 }
@@ -124,9 +127,81 @@ as.col_spec.default <- function(x) {
   stop("`col_types` must be NULL, a list or a string", call. = FALSE)
 }
 
+type_to_col <- function(x, ...) {
+  UseMethod("type_to_col")
+}
+
 #' @export
-print.col_spec <- function(x, n = Inf, condense = NULL, ...) {
-  cat(format.col_spec(x, n = n, condense = condense, ...))
+type_to_col.default <- function(x, ...) {
+  col_character()
+}
+
+#' @export
+type_to_col.logical <- function(x, ...) {
+  col_logical()
+}
+
+#' @export
+type_to_col.integer <- function(x, ...) {
+  col_integer()
+}
+
+#' @export
+type_to_col.double <- function(x, ...) {
+  col_double()
+}
+
+#' @export
+type_to_col.factor <- function(x, ...) {
+  col_factor(levels = levels(x), ordered = is.ordered(x), include_na = any(is.na(levels(x))))
+}
+
+#' @export
+type_to_col.Date <- function(x, ...) {
+  col_date()
+}
+
+#' @export
+type_to_col.POSIXct <- function(x, ...) {
+  col_datetime()
+}
+
+#' @export
+type_to_col.hms <- function(x, ...) {
+  col_time()
+}
+
+#' @export
+as.col_spec.data.frame <- function(x) {
+  as.col_spec(lapply(x, type_to_col))
+}
+
+col_to_short <- function(x, ...) {
+  switch(class(x)[[1]],
+    collector_character = "c",
+    collector_date = "D",
+    collector_datetime = "T",
+    collector_double = "d",
+    collector_factor = "f",
+    collector_guess = "?",
+    collector_integer = "i",
+    collector_logical = "l",
+    collector_number = "n",
+    collector_skip = "-",
+    collector_time = "t"
+  )
+}
+
+#' @export
+as.character.col_spec <- function(x, ...) {
+  paste0(collapse = "",
+    vapply(x$cols, col_to_short, character(1))
+  )
+}
+
+#' @export
+print.col_spec <- function(x, n = Inf, condense = NULL, colour = crayon::has_color(), ...) {
+  cat(format.col_spec(x, n = n, condense = condense, colour = colour, ...))
 
   invisible(x)
 }
@@ -148,7 +223,7 @@ cols_condense <- function(x) {
 }
 
 #' @export
-format.col_spec <- function(x, n = Inf, condense = NULL, ...) {
+format.col_spec <- function(x, n = Inf, condense = NULL, colour = crayon::has_color(), ...) {
 
   if (n == 0) {
     return("")
@@ -181,7 +256,10 @@ format.col_spec <- function(x, n = Inf, condense = NULL, ...) {
         args <- vapply(cols[[i]], deparse2, character(1), sep = "\n    ")
         args <- paste(names(args), args, sep = " = ", collapse = ", ")
 
-        col_names <- names(cols)[[i]]
+        col_funs <- paste0(col_funs, "(", args, ")")
+        col_funs <- colourise_cols(col_funs, colour)
+
+        col_names <- names(cols)[[i]] %||% ""
 
         # Need to handle unnamed columns and columns with non-syntactic names
         named <- col_names != ""
@@ -189,8 +267,8 @@ format.col_spec <- function(x, n = Inf, condense = NULL, ...) {
         non_syntactic <- !is_syntactic(col_names) & named
         col_names[non_syntactic] <- paste0("`", gsub("`", "\\\\`", col_names[non_syntactic]), "`")
 
-        out <- paste0(col_names, " = ", col_funs, "(", args, ")")
-        out[!named] <- paste0(col_funs, "(", args, ")")
+        out <- paste0(col_names, " = ", col_funs)
+        out[!named] <- col_funs[!named]
         out
       },
       character(1)
@@ -211,14 +289,46 @@ format.col_spec <- function(x, n = Inf, condense = NULL, ...) {
   out
 }
 
+colourise_cols <- function(cols, colourise = crayon::has_color()) {
+
+  if (!isTRUE(colourise)) {
+    return(cols)
+  }
+
+  fname <- sub("[(].*", "", cols)
+  for (i in seq_along(cols)) {
+    cols[[i]] <- switch(fname,
+      col_skip = ,
+      col_guess = cols[[i]],
+
+      col_character = ,
+      col_factor = crayon::red(cols[[i]]),
+
+      col_logical = crayon::yellow(cols[[i]]),
+
+      col_double = ,
+      col_integer = ,
+      col_number = crayon::green(cols[[i]]),
+
+      col_date = ,
+      col_datetime = ,
+      col_time = crayon::blue(cols[[i]])
+      )
+  }
+  cols
+}
+
 # Used in read_delim(), read_fwf() and type_convert()
 show_cols_spec <- function(spec, n = getOption("readr.num_columns", 20)) {
   if (n > 0) {
-    message("Parsed with column specification:\n",
-      format(spec, n = n, condense = NULL), appendLF = FALSE)
+    cli_block(class = "readr_spec_message", {
+    cli::cli_h1("Column specification")
+    txt <- strsplit(format(spec, n = n, condense = NULL), "\n")[[1]]
+    cli::cli_verbatim(txt)
     if (length(spec$cols) >= n) {
-      message("See spec(...) for full column specifications.")
+      cli::cli_alert_info("Use {.fn spec} for the full column specifications.")
     }
+  })
   }
 }
 
@@ -264,12 +374,12 @@ col_concise <- function(x) {
     "-" = col_skip(),
     "?" = col_guess(),
     c = col_character(),
-    D = col_date(),
-    d = col_double(),
     f = col_factor(),
+    d = col_double(),
     i = col_integer(),
     l = col_logical(),
     n = col_number(),
+    D = col_date(),
     T = col_datetime(),
     t = col_time(),
     stop("Unknown shortcut: ", x, call. = FALSE)
