@@ -10,11 +10,11 @@ read_lines_chunked <- function(file, callback, chunk_size = 10000, skip = 0,
   if (empty_file(file)) {
     return(character())
   }
-  ds <- datasource(file, skip = skip)
+  ds <- datasource(file, skip = skip, skip_empty_rows = FALSE)
   callback <- as_chunk_callback(callback)
   on.exit(callback$finally(), add = TRUE)
 
-  read_lines_chunked_(ds, locale, na, chunk_size, callback, progress)
+  read_lines_chunked_(ds, locale, na, chunk_size, callback, FALSE, progress)
 
   return(callback$result())
 }
@@ -27,7 +27,7 @@ read_lines_raw_chunked <- function(file, callback, chunk_size = 10000, skip = 0,
   if (empty_file(file)) {
     return(character())
   }
-  ds <- datasource(file, skip = skip)
+  ds <- datasource(file, skip = skip, skip_empty_rows = FALSE)
   callback <- as_chunk_callback(callback)
   on.exit(callback$finally(), add = TRUE)
 
