@@ -1,10 +1,9 @@
-#include <Rcpp.h>
-
 #include "Collector.h"
 #include "Progress.h"
 #include "Source.h"
 
-using namespace Rcpp;
+#include "cpp11/list.hpp"
+#include "cpp11/strings.hpp"
 
 class Reader {
 public:
@@ -12,33 +11,28 @@ public:
       SourcePtr source,
       TokenizerPtr tokenizer,
       std::vector<CollectorPtr> collectors,
-      bool progress = true,
-      CharacterVector colNames = CharacterVector());
+      bool progress,
+      cpp11::strings colNames = cpp11::strings());
 
   Reader(
       SourcePtr source,
       TokenizerPtr tokenizer,
       CollectorPtr collector,
-      bool progress = true,
-      CharacterVector colNames = CharacterVector());
+      bool progress,
+      cpp11::strings colNames = cpp11::strings());
 
-  RObject readToDataFrame(int lines = -1);
-  RObject meltToDataFrame(List locale_, int lines = -1);
+  cpp11::sexp readToDataFrame(int lines = -1);
+  cpp11::sexp meltToDataFrame(cpp11::list locale_, int lines = -1);
 
   template <typename T> T readToVector(int lines) {
     read(lines);
 
-    T out = as<T>(collectors_[0]->vector());
+    SEXP x = collectors_[0]->vector();
+    T out(x);
     collectorsClear();
     return out;
   }
 
-  template <typename T> RObject readToVectorWithWarnings(int lines) {
-    read(lines);
-
-    return warnings_.addAsAttribute(as<T>(collectors_[0]->vector()));
-  }
-
 private:
   Warnings warnings_;
   SourcePtr source_;
@@ -47,15 +41,15 @@ private:
   bool progress_;
   Progress progressBar_;
   std::vector<int> keptColumns_;
-  CharacterVector outNames_;
+  cpp11::writable::strings outNames_;
   bool begun_;
   Token t_;
 
   const static int progressStep_ = 10000;
 
-  void init(CharacterVector colNames);
+  void init(cpp11::strings colNames);
   int read(int lines = -1);
-  int melt(List locale_, int lines = -1);
+  int melt(cpp11::list locale_, int lines = -1);
   void checkColumns(int i, int j, int n);
 
   void collectorsResize(int n);
