@@ -1,5 +1,7 @@
-#include <Rcpp.h>
-using namespace Rcpp;
+#include "cpp11/R.hpp"
+#include "cpp11/integers.hpp"
+#include "cpp11/list.hpp"
+#include "cpp11/sexp.hpp"
 
 #include "Collector.h"
 #include "LocaleInfo.h"
@@ -8,8 +10,8 @@ using namespace Rcpp;
 #include "TokenizerLine.h"
 #include "Warnings.h"
 
-// [[Rcpp::export]]
-IntegerVector dim_tokens_(List sourceSpec, List tokenizerSpec) {
+[[cpp11::register]] cpp11::integers
+dim_tokens_(cpp11::list sourceSpec, cpp11::list tokenizerSpec) {
   SourcePtr source = Source::create(sourceSpec);
   TokenizerPtr tokenizer = Tokenizer::create(tokenizerSpec);
   tokenizer->tokenize(source->begin(), source->end());
@@ -24,11 +26,15 @@ IntegerVector dim_tokens_(List sourceSpec, List tokenizerSpec) {
       cols = t.col();
   }
 
-  return IntegerVector::create(rows + 1, cols + 1);
+  cpp11::writable::integers out(rows + 1);
+  for (auto&& x : out) {
+    x = cols + 1;
+  }
+  return out;
 }
 
-// [[Rcpp::export]]
-std::vector<int> count_fields_(List sourceSpec, List tokenizerSpec, int n_max) {
+[[cpp11::register]] std::vector<int>
+count_fields_(cpp11::list sourceSpec, cpp11::list tokenizerSpec, int n_max) {
   SourcePtr source = Source::create(sourceSpec);
   TokenizerPtr tokenizer = Tokenizer::create(tokenizerSpec);
   tokenizer->tokenize(source->begin(), source->end());
@@ -50,8 +56,8 @@ std::vector<int> count_fields_(List sourceSpec, List tokenizerSpec, int n_max) {
   return fields;
 }
 
-// [[Rcpp::export]]
-RObject guess_header_(List sourceSpec, List tokenizerSpec, List locale_) {
+[[cpp11::register]] cpp11::list guess_header_(
+    cpp11::list sourceSpec, cpp11::list tokenizerSpec, cpp11::list locale_) {
   Warnings warnings;
   LocaleInfo locale(locale_);
   SourcePtr source = Source::create(sourceSpec);
@@ -61,14 +67,21 @@ RObject guess_header_(List sourceSpec, List tokenizerSpec, List locale_) {
 
   CollectorCharacter out(&locale.encoder_);
   out.setWarnings(&warnings);
+  Token t = tokenizer->nextToken();
+  size_t row_num = t.row();
 
-  for (Token t = tokenizer->nextToken(); t.type() != TOKEN_EOF;
+  size_t max_size = 0;
+  size_t capacity = 0;
+
+  for (; t.type() != TOKEN_EOF && t.row() == row_num;
        t = tokenizer->nextToken()) {
-    if (t.row() > (size_t)0) // only read one row
-      break;
+    if (t.col() >= max_size) {
+      max_size = t.col();
+    }
 
-    if (t.col() >= (size_t)out.size()) {
-      out.resize(t.col() + 1);
+    if (max_size >= capacity) {
+      capacity = (max_size + 1) * 2;
+      out.resize(capacity);
     }
 
     if (t.type() == TOKEN_STRING) {
@@ -76,11 +89,15 @@ RObject guess_header_(List sourceSpec, List tokenizerSpec, List locale_) {
     }
   }
 
-  return out.vector();
+  out.resize(max_size + 1);
+
+  using namespace cpp11::literals;
+  return cpp11::writable::list(
+      {"header"_nm = out.vector(), "skip"_nm = source->skippedRows() + 1});
 }
 
-// [[Rcpp::export]]
-RObject tokenize_(List sourceSpec, List tokenizerSpec, int n_max) {
+[[cpp11::register]] SEXP
+tokenize_(cpp11::list sourceSpec, cpp11::list tokenizerSpec, int n_max) {
   Warnings warnings;
 
   SourcePtr source = Source::create(sourceSpec);
@@ -88,7 +105,7 @@ RObject tokenize_(List sourceSpec, List tokenizerSpec, int n_max) {
   tokenizer->tokenize(source->begin(), source->end());
   tokenizer->setWarnings(&warnings);
 
-  std::vector<std::vector<std::string> > rows;
+  std::vector<std::vector<std::string>> rows;
 
   for (Token t = tokenizer->nextToken(); t.type() != TOKEN_EOF;
        t = tokenizer->nextToken()) {
@@ -106,17 +123,22 @@ RObject tokenize_(List sourceSpec, List tokenizerSpec, int n_max) {
     row[t.col()] = t.asString();
   }
 
-  RObject out = wrap(rows);
+  cpp11::writable::list out;
+  out.reserve(rows.size());
+
+  for (auto&& row : rows) {
+    out.push_back(cpp11::as_sexp(row));
+  }
+
   return warnings.addAsAttribute(out);
 }
 
-// [[Rcpp::export]]
-SEXP parse_vector_(
-    CharacterVector x,
-    List collectorSpec,
-    List locale_,
+[[cpp11::register]] SEXP parse_vector_(
+    cpp11::strings x,
+    cpp11::list collectorSpec,
+    cpp11::list locale_,
     const std::vector<std::string>& na,
-    const bool trim_ws = true) {
+    const bool trim_ws) {
   Warnings warnings;
   int n = x.size();
 
@@ -141,5 +163,5 @@ SEXP parse_vector_(
     col->setValue(i, t);
   }
 
-  return warnings.addAsAttribute(col->vector());
+  return warnings.addAsAttribute(static_cast<SEXP>(col->vector()));
 }
